from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .products_get_response import ProductsGetResponse

class ProductsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/admin/v1/accounts/{accountId}/users/{userId}/products
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new ProductsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/admin/v1/accounts/{accountId}/users/{userId}/products{?fields*,filter%5Bkey%5D*,filter%5BprojectId%5D*,limit*,offset*,sort*}", path_parameters)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[ProductsRequestBuilderGetQueryParameters]] = None) -> Optional[ProductsGetResponse]:
        """
        Returns a list of ACC products the user is associated with in their assigned projects.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[ProductsGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .products_get_response import ProductsGetResponse

        return await self.request_adapter.send_async(request_info, ProductsGetResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[ProductsRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Returns a list of ACC products the user is associated with in their assigned projects.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def with_url(self,raw_url: str) -> ProductsRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: ProductsRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return ProductsRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class ProductsRequestBuilderGetQueryParameters():
        """
        Returns a list of ACC products the user is associated with in their assigned projects.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "filterkey":
                return "filter%5Bkey%5D"
            if original_name == "filterproject_id":
                return "filter%5BprojectId%5D"
            if original_name == "fields":
                return "fields"
            if original_name == "limit":
                return "limit"
            if original_name == "offset":
                return "offset"
            if original_name == "sort":
                return "sort"
            return original_name
        
        # List of fields to return in the response. Defaults to all fields.Possible values: ``projectIds``, ``name`` and ``icon``.
        fields: Optional[list[str]] = None

        # Filters the list of products by product key — a machine-readable identifier for an ACC product (such as ``docs``, ``build``, or ``cost``).You can specify one or more keys to return only those products the user is associated with.Example: ``filter[key]=docs,build``Possible values: ``accountAdministration``, ``autoSpecs``, ``build``, ``buildingConnected``, ``capitalPlanning``, ``cloudWorksharing``, ``cost``, ``designCollaboration``, ``docs``, ``financials``, ``insight``, ``modelCoordination``, ``projectAdministration``, ``takeoff``, and ``workshopxr``.
        filterkey: Optional[list[str]] = None

        # A list of project IDs. Only results where the user is associated with one or more of the specified projects are returned.
        filterproject_id: Optional[list[str]] = None

        # The maximum number of records to return in the response.Default: ``20``Minimum: ``1``Maximum: ``200`` (If a larger value is provided, only 200 records are returned)
        limit: Optional[int] = None

        # The index of the first record to return.Used for pagination in combination with the ``limit`` parameter.Example: ``limit=20`` and ``offset=40`` returns records 41–60.
        offset: Optional[int] = None

        # The list of fields to sort by.Each property can be followed by a direction modifier of either ``asc`` (ascending) or ``desc`` (descending). The default is ``asc``.Possible values:  ``name``.Default is the order in database.
        sort: Optional[list[str]] = None

    
    @dataclass
    class ProductsRequestBuilderGetRequestConfiguration(RequestConfiguration[ProductsRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .classifications_import_post_request_body_classifications import ClassificationsImportPostRequestBody_classifications

@dataclass
class ClassificationsImportPostRequestBody(Parsable):
    # The classification system name.Max length: 200
    classification_system_name: Optional[str] = None
    # The classification hierarchy.The classification hierarchy is configured as a JSON array in the payload, created from a spreadsheet file.Max size: ``30000``.For more details, see the `ACC Configure Takeoff Settings <https://help.autodesk.com/view/TAKEOFF/ENU/?guid=Configure_Takeoff_Settings>`_ help documentation.
    classifications: Optional[list[ClassificationsImportPostRequestBody_classifications]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ClassificationsImportPostRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ClassificationsImportPostRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ClassificationsImportPostRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .classifications_import_post_request_body_classifications import ClassificationsImportPostRequestBody_classifications

        from .classifications_import_post_request_body_classifications import ClassificationsImportPostRequestBody_classifications

        fields: dict[str, Callable[[Any], None]] = {
            "classificationSystemName": lambda n : setattr(self, 'classification_system_name', n.get_str_value()),
            "classifications": lambda n : setattr(self, 'classifications', n.get_collection_of_object_values(ClassificationsImportPostRequestBody_classifications)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("classificationSystemName", self.classification_system_name)
        writer.write_collection_of_object_values("classifications", self.classifications)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .me_get_response_external_users import MeGetResponse_externalUsers
    from .me_get_response_permitted_actions import MeGetResponse_permittedActions
    from .me_get_response_user import MeGetResponse_user
    from .me_get_response_workflow import MeGetResponse_workflow

@dataclass
class MeGetResponse(Parsable):
    # The ID of the default RFI type assigned to the project. This is the unique identifier of the RFI type that will be selected by default when creating a new RFI.
    default_rfi_type: Optional[UUID] = None
    # Not relevant
    external_users: Optional[list[MeGetResponse_externalUsers]] = None
    # Not relevant
    maintenance_end_date: Optional[str] = None
    # The list of actions that are permitted for the user.
    permitted_actions: Optional[list[MeGetResponse_permittedActions]] = None
    # The current user’s details.
    user: Optional[MeGetResponse_user] = None
    # The user’s assigned workflow roles and workflow type for RFIs in the current project.
    workflow: Optional[list[MeGetResponse_workflow]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> MeGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: MeGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return MeGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .me_get_response_external_users import MeGetResponse_externalUsers
        from .me_get_response_permitted_actions import MeGetResponse_permittedActions
        from .me_get_response_user import MeGetResponse_user
        from .me_get_response_workflow import MeGetResponse_workflow

        from .me_get_response_external_users import MeGetResponse_externalUsers
        from .me_get_response_permitted_actions import MeGetResponse_permittedActions
        from .me_get_response_user import MeGetResponse_user
        from .me_get_response_workflow import MeGetResponse_workflow

        fields: dict[str, Callable[[Any], None]] = {
            "defaultRfiType": lambda n : setattr(self, 'default_rfi_type', n.get_uuid_value()),
            "externalUsers": lambda n : setattr(self, 'external_users', n.get_collection_of_object_values(MeGetResponse_externalUsers)),
            "maintenanceEndDate": lambda n : setattr(self, 'maintenance_end_date', n.get_str_value()),
            "permittedActions": lambda n : setattr(self, 'permitted_actions', n.get_collection_of_object_values(MeGetResponse_permittedActions)),
            "user": lambda n : setattr(self, 'user', n.get_object_value(MeGetResponse_user)),
            "workflow": lambda n : setattr(self, 'workflow', n.get_collection_of_object_values(MeGetResponse_workflow)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_uuid_value("defaultRfiType", self.default_rfi_type)
        writer.write_collection_of_object_values("externalUsers", self.external_users)
        writer.write_str_value("maintenanceEndDate", self.maintenance_end_date)
        writer.write_collection_of_object_values("permittedActions", self.permitted_actions)
        writer.write_object_value("user", self.user)
        writer.write_collection_of_object_values("workflow", self.workflow)
    




from enum import Enum

class LatestGetResponse_status(str, Enum):
    Pending = "Pending",
    Processing = "Processing",
    Successful = "Successful",
    Partial = "Partial",
    Failed = "Failed",




from enum import Enum

class GetAssociationTypeQueryParameterType(str, Enum):
    Contract = "Contract",
    Payment = "Payment",
    BudgetPayment = "BudgetPayment",
    CostPayment = "CostPayment",
    Expense = "Expense",
    PCO = "PCO",
    OCO = "OCO",
    SCO = "SCO",
    RCO = "RCO",
    RFQ = "RFQ",
    DistributionItem = "DistributionItem",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class StoragePostResponse(Parsable):
    # The URN of the storage.Note that in Sheets each project has its own OSS bucket,it is recommended to extract bucket key from storage URN every time before you call OSS APIs.
    urn: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> StoragePostResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: StoragePostResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return StoragePostResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "urn": lambda n : setattr(self, 'urn', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("urn", self.urn)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .item.with_version_item_request_builder import WithVersionItemRequestBuilder

class VersionRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/autospecs/v1/projects/{projectId}/version
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new VersionRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/autospecs/v1/projects/{projectId}/version", path_parameters)
    
    def by_version_id(self,version_id: str) -> WithVersionItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.construction.autospecs.v1.projects.item.version.item collection
        param version_id: The AutoSpecs version ID of the project. For information about how to find the version ID, see the first few steps of the `Retrieve Submittal Log </en/docs/acc/v1/tutorials/autospecs/upload-document/>`_ tutorial.
        Returns: WithVersionItemRequestBuilder
        """
        if version_id is None:
            raise TypeError("version_id cannot be null.")
        from .item.with_version_item_request_builder import WithVersionItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["versionId"] = version_id
        return WithVersionItemRequestBuilder(self.request_adapter, url_tpl_params)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .expenses_get_response import ExpensesGetResponse
    from .expenses_post_request_body import ExpensesPostRequestBody
    from .expenses_post_response import ExpensesPostResponse
    from .item.expense_item_request_builder import ExpenseItemRequestBuilder

class ExpensesRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /cost/v1/containers/{containerId}/expenses
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new ExpensesRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/cost/v1/containers/{containerId}/expenses{?filter%5BbudgetPaymentId%5D*,filter%5BcreatedAt%5D*,filter%5BexternalId%5D*,filter%5BexternalSystem%5D*,filter%5Bid%5D*,filter%5BlastModifiedSince%5D*,filter%5BmainContractId%5D*,filter%5Bnumber%5D*,filter%5Bstatus%5D*,include*,limit*,offset*,sort*}", path_parameters)
    
    def by_expense_id(self,expense_id: str) -> ExpenseItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.cost.v1.containers.item.expenses.item collection
        param expense_id: The object ID of the expense. You can obtain this ID from the response to the `POST expenses </en/docs/bim360/v1/reference/http/cost-expenses-POST/>`_ or `GET expenses </en/docs/bim360/v1/reference/http/cost-expenses-GET/>`_ endpoint.
        Returns: ExpenseItemRequestBuilder
        """
        if expense_id is None:
            raise TypeError("expense_id cannot be null.")
        from .item.expense_item_request_builder import ExpenseItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["expense%2Did"] = expense_id
        return ExpenseItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[ExpensesRequestBuilderGetQueryParameters]] = None) -> Optional[ExpensesGetResponse]:
        """
        Retrieves the requested set of expenses in the specified project.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[ExpensesGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .expenses_get_response import ExpensesGetResponse

        return await self.request_adapter.send_async(request_info, ExpensesGetResponse, None)
    
    async def post(self,body: ExpensesPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[ExpensesPostResponse]:
        """
        Creates an expense in the given project.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[ExpensesPostResponse]
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_post_request_information(
            body, request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .expenses_post_response import ExpensesPostResponse

        return await self.request_adapter.send_async(request_info, ExpensesPostResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[ExpensesRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Retrieves the requested set of expenses in the specified project.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_post_request_information(self,body: ExpensesPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        Creates an expense in the given project.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.POST, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> ExpensesRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: ExpensesRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return ExpensesRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class ExpensesRequestBuilderGetQueryParameters():
        """
        Retrieves the requested set of expenses in the specified project.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "filterbudget_payment_id":
                return "filter%5BbudgetPaymentId%5D"
            if original_name == "filtercreated_at":
                return "filter%5BcreatedAt%5D"
            if original_name == "filterexternal_id":
                return "filter%5BexternalId%5D"
            if original_name == "filterexternal_system":
                return "filter%5BexternalSystem%5D"
            if original_name == "filterid":
                return "filter%5Bid%5D"
            if original_name == "filterlast_modified_since":
                return "filter%5BlastModifiedSince%5D"
            if original_name == "filtermain_contract_id":
                return "filter%5BmainContractId%5D"
            if original_name == "filternumber":
                return "filter%5Bnumber%5D"
            if original_name == "filterstatus":
                return "filter%5Bstatus%5D"
            if original_name == "include":
                return "include"
            if original_name == "limit":
                return "limit"
            if original_name == "offset":
                return "offset"
            if original_name == "sort":
                return "sort"
            return original_name
        
        # Returns only the payments associated with the budget payments (used to query the related cost payments or expenses) that are identified on this list of IDs. Separate multiple IDs with commas; for example, ``filter[budgetPaymentId]=id1,id2``.
        filterbudget_payment_id: Optional[list[str]] = None

        # Filter data by its create date. This may be an ISO 8601 date string or a range. Ranges can be **lowerValue..upperValue**, **lowerValue..** or **..upperValue**. The range tests are always inclusive of their endpoints.
        filtercreated_at: Optional[str] = None

        # The ID of the item in the external ERP system. Use this ID to track or look up data in an integrated ERP system. For example, ``filter[externalId]=id1,id2``.
        filterexternal_id: Optional[list[str]] = None

        # The name of the external ERP system. Use this name to identify or search within the integrated system. For example, ``filter[externalSystem]=Sage300``.Max length: 255
        filterexternal_system: Optional[str] = None

        # Returns only the items that are identified by the provided list of item IDs. Separate multiple IDs with commas. For example, ``filter[id]=id1,id2``.
        filterid: Optional[list[str]] = None

        # Returns only items that were modified since the specified date and time, in ISO 8601 format. For example, ``filter[lastModifiedSince]=2020-03-01T13:00:00Z``.
        filterlast_modified_since: Optional[str] = None

        # The Main Contract ID. Separate multiple IDs with commas. For example, ``filter[mainContractId]=id1,id2``, or filter these items that are not linked to any main contract  ``filter[mainContractId]=blank``.
        filtermain_contract_id: Optional[list[str]] = None

        # Returns only the items that are identified by the provided list of auto-generated sequence numbers. Separate multiple numbers with commas; for example, ``filter[number]=0001,0002``.
        filternumber: Optional[list[str]] = None

        # Returns only contracts with the specified statuses. Separate multiple values with commas. For example, ``filter[status]=draft,pending``. Possible values:``draft``, ``pending``, ``submitted``, ``revise``, ``sent``, ``signed``, ``executed``, ``closed``, ``inReview``.
        filterstatus: Optional[list[str]] = None

        # A list of the nested expense resources to include in the response with the expenses. For example, ``include=expenseItems`` returns the expense items related to each expense. ``include=attributes`` will return custom attributes which represents the ``properties`` in the response.Possible values: ``expenseItems``, ``mainContract``, ``attributes``, ``externalRelationship``, ``paymentReferences``.
        include: Optional[list[str]] = None

        # The maximum number of records returned per page. Default: ``100``. A page may contain fewer records than the limit if there are fewer matching items or if it is the last page of results.
        limit: Optional[int] = None

        # The number of records to skip before returning results. Used together with ``limit`` to paginate through results, where ``offset`` specifies the starting point and ``limit`` specifies the number of records to return.
        offset: Optional[int] = None

        # Defines the sort order for the results. Each attribute can be sorted in ``asc`` (default) or ``desc`` order. For example, ``sort=name desc`` sorts the results by name in descending order.
        sort: Optional[str] = None

    
    @dataclass
    class ExpensesRequestBuilderGetRequestConfiguration(RequestConfiguration[ExpensesRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class ExpensesRequestBuilderPostRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class ExportsPostRequestBody_options_photoMarkups(Parsable):
    """
    The list of types of photo markups to export.
    """
    # ``true``: published photo markups will be included in the exported sheets.``false``: (default) published photo markups will not be included in the exported sheets.
    include_published_markups: Optional[bool] = None
    # ``true``: include photo markups in the exported sheets that are visible only to their creators or assignees.``false``: (default) export sheets without including unpublished photo markups.
    include_unpublished_markups: Optional[bool] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ExportsPostRequestBody_options_photoMarkups:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ExportsPostRequestBody_options_photoMarkups
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ExportsPostRequestBody_options_photoMarkups()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "includePublishedMarkups": lambda n : setattr(self, 'include_published_markups', n.get_bool_value()),
            "includeUnpublishedMarkups": lambda n : setattr(self, 'include_unpublished_markups', n.get_bool_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_bool_value("includePublishedMarkups", self.include_published_markups)
        writer.write_bool_value("includeUnpublishedMarkups", self.include_unpublished_markups)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .assets_get_response_results_custom_attributes import AssetsGetResponse_results_customAttributes

@dataclass
class AssetsGetResponse_results(Parsable):
    # A string that lists a barcode value that may be assigned to the asset. The string uses whatever format your barcode system supports.
    barcode: Optional[str] = None
    # The ID of the category to which the asset belongs.
    category_id: Optional[str] = None
    # An identifying name for the asset such as ``laundry 220V outlet``. The name need not be unique, and shouldn't be confused with the asset 'id' field, which is created and assigned when the asset is created. This value appears as ``Asset ID`` in the Assets UI, and may sometimes be called ``Name`` in asset exports.
    client_asset_id: Optional[str] = None
    # The ID of the company to which the creating user belongs in the project.
    company_id: Optional[UUID] = None
    # The time when the component was created (ISO8601 Date time format in UTC).
    created_at: Optional[str] = None
    # The actor that created the component. This is an Autodesk / Oxygen ID.
    created_by: Optional[str] = None
    # An optional JSON dictionary specifying one or more custom attributes and values to be assigned to theasset. The custom attributes must belong to the set of custom attributes specified by the asset's category.The dictionary is a set of key:value pairs that each starts with the unique custom attribute name (not tobe confused with the custom attribute's display name) followed by the attribute value. The value must usethe data type defined by the attribute:For ``text``, the value is a string.For ``date``, the value is an ISO8601 date string with no time, for example, ``2020-04-10``.For ``select``, the value is a valid ID from the list of values defined for this custom attribute.For ``multi-select``, the value is an array of valid IDs from the list of values defined for this custom attribute.For ``boolean``, the value is a boolean.For ``numeric``, the value is a string that parses as a valid floating point number (not localized).
    custom_attributes: Optional[AssetsGetResponse_results_customAttributes] = None
    # The time when the component was deleted at (ISO8601 Date time format in UTC).
    deleted_at: Optional[str] = None
    # The actor that deleted the component. This is an Autodesk / Oxygen ID.
    deleted_by: Optional[str] = None
    # A brief description of the asset. Currently limited to ``1000`` characters.
    description: Optional[str] = None
    # The ID of the component.
    id: Optional[str] = None
    # A flag indicating whether the component is active or inactive (``isActive`` is ``true`` if-and-only-if ``deletedAt`` is empty).
    is_active: Optional[bool] = None
    # The ID of the location of the asset. This value is supplied through the Locations API.
    location_id: Optional[UUID] = None
    # The ID of the status assigned to the asset. The status must belong to the status set specified by the asset's category.
    status_id: Optional[UUID] = None
    # The time when the component was last updated (ISO8601 Date time format in UTC).
    updated_at: Optional[str] = None
    # The actor that last updated the component. This is an Autodesk / Oxygen ID.
    updated_by: Optional[str] = None
    # A global sequence number that is incremented any time a component of this type is created, updated, or deleted. If you cache components, you can use the version value to compare the cached component to the same component online to see if the component has been updated. If the online component has a higher version value, it has been updated.
    version: Optional[int] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> AssetsGetResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: AssetsGetResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return AssetsGetResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .assets_get_response_results_custom_attributes import AssetsGetResponse_results_customAttributes

        from .assets_get_response_results_custom_attributes import AssetsGetResponse_results_customAttributes

        fields: dict[str, Callable[[Any], None]] = {
            "barcode": lambda n : setattr(self, 'barcode', n.get_str_value()),
            "categoryId": lambda n : setattr(self, 'category_id', n.get_str_value()),
            "clientAssetId": lambda n : setattr(self, 'client_asset_id', n.get_str_value()),
            "companyId": lambda n : setattr(self, 'company_id', n.get_uuid_value()),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_str_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "customAttributes": lambda n : setattr(self, 'custom_attributes', n.get_object_value(AssetsGetResponse_results_customAttributes)),
            "deletedAt": lambda n : setattr(self, 'deleted_at', n.get_str_value()),
            "deletedBy": lambda n : setattr(self, 'deleted_by', n.get_str_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "isActive": lambda n : setattr(self, 'is_active', n.get_bool_value()),
            "locationId": lambda n : setattr(self, 'location_id', n.get_uuid_value()),
            "statusId": lambda n : setattr(self, 'status_id', n.get_uuid_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_str_value()),
            "updatedBy": lambda n : setattr(self, 'updated_by', n.get_str_value()),
            "version": lambda n : setattr(self, 'version', n.get_int_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("barcode", self.barcode)
        writer.write_str_value("categoryId", self.category_id)
        writer.write_str_value("clientAssetId", self.client_asset_id)
        writer.write_uuid_value("companyId", self.company_id)
        writer.write_str_value("createdAt", self.created_at)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_object_value("customAttributes", self.custom_attributes)
        writer.write_str_value("deletedAt", self.deleted_at)
        writer.write_str_value("deletedBy", self.deleted_by)
        writer.write_str_value("description", self.description)
        writer.write_str_value("id", self.id)
        writer.write_bool_value("isActive", self.is_active)
        writer.write_uuid_value("locationId", self.location_id)
        writer.write_uuid_value("statusId", self.status_id)
        writer.write_str_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedBy", self.updated_by)
        writer.write_int_value("version", self.version)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class ItemsValidateCustomIdentifierPostRequestBody(Parsable):
    # The ``customIdentifier`` is used to ensure that the chosen identifier adheres to format rules and is unique within the project. Submittal managers can assign custom numbers manually or automatically.When validating a custom identifier, consider the following rules:- Ensure the identifier is unique within the project.- For global sequences, ensure it is unique within the entire project.- For spec sequences, ensure it is unique within the specific spec section.- Spec sequence is in the format ``<spec_identifier>-<sequential_number>``. You only need to specify the sequential number. The full number - ``<spec_identifier>-<sequential_number>`` - appears in the response payload in the ``customIdentifierHumanReadable`` attribute.Regardless of whether the project uses a global or spec sequence, you should always provide only the sequential number portion without the spec ID when sending the ``customIdentifier``.For more information on custom numbering, see the `Help documentation <https://help.autodesk.com/view/BUILD/ENU/?guid=Submittal_Custom_Numbering>`_.
    custom_identifier: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ItemsValidateCustomIdentifierPostRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ItemsValidateCustomIdentifierPostRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ItemsValidateCustomIdentifierPostRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "customIdentifier": lambda n : setattr(self, 'custom_identifier', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("customIdentifier", self.custom_identifier)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID
from warnings import warn

if TYPE_CHECKING:
    from .get_filter_status_query_parameter_type import GetFilterStatusQueryParameterType
    from .item.with_workflow_item_request_builder import WithWorkflowItemRequestBuilder
    from .workflows_get_response import WorkflowsGetResponse
    from .workflows_post_request_body import WorkflowsPostRequestBody
    from .workflows_post_response import WorkflowsPostResponse

class WorkflowsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/reviews/v1/projects/{projectId}/workflows
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new WorkflowsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/reviews/v1/projects/{projectId}/workflows{?filter%5Binitiator%5D*,filter%5Bstatus%5D*,limit*,offset*,sort*}", path_parameters)
    
    def by_workflow_id(self,workflow_id: UUID) -> WithWorkflowItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.construction.reviews.v1.projects.item.workflows.item collection
        param workflow_id: The ID of the approval workflow.To find available workflow IDs, call `GET Workflows </en/docs/acc/v1/reference/http/reviews-workflows-GET>`_.
        Returns: WithWorkflowItemRequestBuilder
        """
        if workflow_id is None:
            raise TypeError("workflow_id cannot be null.")
        from .item.with_workflow_item_request_builder import WithWorkflowItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["workflowId"] = workflow_id
        return WithWorkflowItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[WorkflowsRequestBuilderGetQueryParameters]] = None) -> Optional[WorkflowsGetResponse]:
        """
        Retrieves all approval workflows used for file reviews in a given project.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[WorkflowsGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .workflows_get_response import WorkflowsGetResponse

        return await self.request_adapter.send_async(request_info, WorkflowsGetResponse, None)
    
    async def post(self,body: WorkflowsPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[WorkflowsPostResponse]:
        """
        Creates a new approval workflow in the specified project.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[WorkflowsPostResponse]
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_post_request_information(
            body, request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .workflows_post_response import WorkflowsPostResponse

        return await self.request_adapter.send_async(request_info, WorkflowsPostResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[WorkflowsRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Retrieves all approval workflows used for file reviews in a given project.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_post_request_information(self,body: WorkflowsPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        Creates a new approval workflow in the specified project.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.POST, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> WorkflowsRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: WorkflowsRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return WorkflowsRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class WorkflowsRequestBuilderGetQueryParameters():
        """
        Retrieves all approval workflows used for file reviews in a given project.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "filterinitiator":
                return "filter%5Binitiator%5D"
            if original_name == "filterstatus":
                return "filter%5Bstatus%5D"
            if original_name == "limit":
                return "limit"
            if original_name == "offset":
                return "offset"
            if original_name == "sort":
                return "sort"
            return original_name
        
        # Filters the results based on who initiated the workflow. For example: ``filter[initiator]=true``.``true``: return only workflows initiated by the current user.This filter is ignored if the user is a project admin.``false``: (default) return workflows regardless of who initiated them.Note that this filter cannot be used together with ``filter[status]``.
        filterinitiator: Optional[bool] = None

        # Filters the results by workflow status. For example: ``filter[status]=INACTIVE``. Possible values:``ACTIVE``: return only active workflows.``INACTIVE``: return only inactive (disabled) workflows.Default: ``ACTIVE``.Note that this filter cannot be used together with ``filter[initiator]``.
        filterstatus: Optional[GetFilterStatusQueryParameterType] = None

        # The maximum number of approval workflows to return in a single request. Possible values: ``1-50``. Maximum: ``50``. Default: ``50``. For example: ``limit=2``.
        limit: Optional[int] = None

        # The index at which the endpoint starts returning results. Used for pagination. Default: ``0``. For example: ``offset=10``.
        offset: Optional[int] = None

        # Specifies a single field to sort the results by. The default order is ascending (``asc``); to sort in descending order, add ``desc``.Possible sorting fields: ``name``, ``status``, ``updatedAt``.For example: ``sort=name desc``.
        sort: Optional[str] = None

    
    @dataclass
    class WorkflowsRequestBuilderGetRequestConfiguration(RequestConfiguration[WorkflowsRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class WorkflowsRequestBuilderPostRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .projects_get_response_results_project_value_currency import ProjectsGetResponse_results_projectValue_currency

@dataclass
class ProjectsGetResponse_results_projectValue(Parsable):
    """
    Contains details about the estimated cost of the project, including the amount (``value``) and the currency (``currency``).
    """
    from .projects_get_response_results_project_value_currency import ProjectsGetResponse_results_projectValue_currency

    # The currency of the project value. Default: ``USD``.Possible values: ``AED``, ``AFN``, ``ALL``, ``AMD``, ``ANG``, ``AOA``, ``ARS``, ``AUD``, ``AWG``, ``AZN``, ``BAM``, ``BBD``, ``BDT``, ``BGN``, ``BHD``, ``BIF``, ``BMD``, ``BND``, ``BOB``, ``BOV``, ``BRL``, ``BSD``, ``BTN``, ``BWP``, ``BYN``, ``BYR``, ``BZD``, ``CAD``, ``CDF``, ``CHE``, ``CHF``, ``CHW``, ``CLF``, ``CLP``, ``CNY``, ``COP``, ``COU``, ``CRC``, ``CUC``, ``CUP``, ``CVE``, ``CZK``, ``DJF``, ``DKK``, ``DOP``, ``DZD``, ``EEK``, ``EGP``, ``ERN``, ``ETB``, ``EUR``, ``FJD``, ``FKP``, ``GBP``, ``GEL``, ``GHS``, ``GIP``, ``GMD``, ``GNF``, ``GTQ``, ``GYD``, ``HKD``, ``HNL``, ``HRK``, ``HTG``, ``HUF``, ``IDR``, ``ILS``, ``INR``, ``IQD``, ``IRR``, ``ISK``, ``JMD``, ``JOD``, ``JPY``, ``KES``, ``KGS``, ``KHR``, ``KMF``, ``KPW``, ``KRW``, ``KWD``, ``KYD``, ``KZT``, ``LAK``, ``LBP``, ``LKR``, ``LRD``, ``LSL``, ``LTL``, ``LVL``, ``LYD``, ``MAD``, ``MDL``, ``MGA``, ``MKD``, ``MMK``, ``MNT``, ``MOP``, ``MRU``, ``MUR``, ``MVR``, ``MWK``, ``MXN``, ``MXV``, ``MYR``, ``MZN``, ``NAD``, ``NGN``, ``NIO``, ``NOK``, ``NPR``, ``NZD``, ``OMR``, ``PAB``, ``PEN``, ``PGK``, ``PHP``, ``PKR``, ``PLN``, ``PYG``, ``QAR``, ``RON``, ``RSD``, ``RUB``, ``RWF``, ``SAR``, ``SBD``, ``SCR``, ``SDG``, ``SEK``, ``SGD``, ``SHP``, ``SLE``, ``SLL``, ``SOS``, ``SRD``, ``SSP``, ``STN``, ``SVC``, ``SYP``, ``SZL``, ``THB``, ``TJS``, ``TMT``, ``TND``, ``TOP``, ``TRL``, ``TRY``, ``TTD``, ``TWD``, ``TZS``, ``UAH``, ``UGX``, ``USD``, ``USN``, ``UYI``, ``UYU``, ``UYW``, ``UZS``, ``VED``, ``VES``, ``VND``, ``VUV``, ``WST``, ``XAF``, ``XAG``, ``XAU``, ``XBA``, ``XBB``, ``XBC``, ``XBD``, ``XCD``, ``XDR``, ``XOF``, ``XPD``, ``XPF``, ``XPT``, ``XSU``, ``XTS``, ``XUA``, ``XXX``, ``YER``, ``ZAR``, ``ZMW``, ``ZWL``
    currency: Optional[ProjectsGetResponse_results_projectValue_currency] = ProjectsGetResponse_results_projectValue_currency("USD")
    # The estimated cost of the project, based on the ``currency`` specified in the currency field. Default: ``0``.
    value: Optional[float] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ProjectsGetResponse_results_projectValue:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ProjectsGetResponse_results_projectValue
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ProjectsGetResponse_results_projectValue()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .projects_get_response_results_project_value_currency import ProjectsGetResponse_results_projectValue_currency

        from .projects_get_response_results_project_value_currency import ProjectsGetResponse_results_projectValue_currency

        fields: dict[str, Callable[[Any], None]] = {
            "currency": lambda n : setattr(self, 'currency', n.get_enum_value(ProjectsGetResponse_results_projectValue_currency)),
            "value": lambda n : setattr(self, 'value', n.get_float_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_enum_value("currency", self.currency)
        writer.write_float_value("value", self.value)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .metadata_get_response_versions_status import MetadataGetResponse_versions_status

@dataclass
class MetadataGetResponse_versions(Parsable):
    # The date and time the version was created in ISO8601 format. For example, ``2023-01-17T09:54:27.000Z``.
    created_at: Optional[str] = None
    # ``true`` if this is the current version``false`` if this is not the current version
    current_version: Optional[bool] = None
    # The ID of the AutoSpecs version, generated by AutoSpecs when creating the new version.
    id: Optional[str] = None
    # The version name provided by the user when creating the version.
    name: Optional[str] = None
    # The status of the specification extraction. Note that Smart Register data (GET smartRegster) is only available when the status is ``Completed``.Note that we do not currently support webhooks for the AutoSpecs API.Possible values: ``Completed``, ``In Progress``, ``Failed``
    status: Optional[MetadataGetResponse_versions_status] = None
    # The last date and time the version was updated in in ISO8601. For example ``2023-01-17T09:54:27.000Z``.
    updated_at: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> MetadataGetResponse_versions:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: MetadataGetResponse_versions
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return MetadataGetResponse_versions()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .metadata_get_response_versions_status import MetadataGetResponse_versions_status

        from .metadata_get_response_versions_status import MetadataGetResponse_versions_status

        fields: dict[str, Callable[[Any], None]] = {
            "createdAt": lambda n : setattr(self, 'created_at', n.get_str_value()),
            "currentVersion": lambda n : setattr(self, 'current_version', n.get_bool_value()),
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(MetadataGetResponse_versions_status)),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("createdAt", self.created_at)
        writer.write_bool_value("currentVersion", self.current_version)
        writer.write_str_value("id", self.id)
        writer.write_str_value("name", self.name)
        writer.write_enum_value("status", self.status)
        writer.write_str_value("updatedAt", self.updated_at)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .forms_post_request_body_assignee_type import FormsPostRequestBody_assigneeType

@dataclass
class FormsPostRequestBody(Parsable):
    # Identifier of the company, role, or user assigned to the form. Note: the assignee must be a contributor of the template.
    assignee_id: Optional[str] = None
    # Type of entity to assign the form to.Possible values: ``company``, ``role``, ``user``
    assignee_type: Optional[FormsPostRequestBody_assigneeType] = None
    # Text for the form description field.Max length: 8000
    description: Optional[str] = None
    # Date the form pertains to, must be after 1950-01-01.
    form_date: Optional[str] = None
    # The unique identifier of the form. If you do not specify an ID, one will be assigned.
    id: Optional[UUID] = None
    # Location associated with the Form. For more information about the location, see `GET nodes </en/docs/acc/v1/reference/http/locations-nodes-GET/>`_.
    location_id: Optional[UUID] = None
    # The name of the form instance. If the specified value is ``null`` or empty, it defaults to the form's template name.Max length: 100
    name: Optional[str] = None
    # Text for the form's notes section.Max length: 8000
    notes: Optional[str] = None
    # Date and time indicating when form was created on the client or external system (optional).
    user_created_at: Optional[datetime.datetime] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> FormsPostRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: FormsPostRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return FormsPostRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .forms_post_request_body_assignee_type import FormsPostRequestBody_assigneeType

        from .forms_post_request_body_assignee_type import FormsPostRequestBody_assigneeType

        fields: dict[str, Callable[[Any], None]] = {
            "assigneeId": lambda n : setattr(self, 'assignee_id', n.get_str_value()),
            "assigneeType": lambda n : setattr(self, 'assignee_type', n.get_enum_value(FormsPostRequestBody_assigneeType)),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "formDate": lambda n : setattr(self, 'form_date', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "locationId": lambda n : setattr(self, 'location_id', n.get_uuid_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "notes": lambda n : setattr(self, 'notes', n.get_str_value()),
            "userCreatedAt": lambda n : setattr(self, 'user_created_at', n.get_datetime_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("assigneeId", self.assignee_id)
        writer.write_enum_value("assigneeType", self.assignee_type)
        writer.write_str_value("description", self.description)
        writer.write_str_value("formDate", self.form_date)
        writer.write_uuid_value("id", self.id)
        writer.write_uuid_value("locationId", self.location_id)
        writer.write_str_value("name", self.name)
        writer.write_str_value("notes", self.notes)
        writer.write_datetime_value("userCreatedAt", self.user_created_at)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .permissions_batch_create_post_response_results_subject_type import PermissionsBatchCreatePostResponse_results_subjectType

@dataclass
class PermissionsBatchCreatePostResponse_results(Parsable):
    # Permitted actions for the user, role, or company.The permission action group is different in BIM 360 Document Management and ACC Build Files.- The six permission levels in BIM 360 Document Management correspond to one or more actions:- View Only: ``VIEW``, ``COLLABORATE``- View/Download: ``VIEW``, ``DOWNLOAD``, ``COLLABORATE``- Upload Only: ``PUBLISH``- View/Download+Upload: ``PUBLISH``, ``VIEW``, ``DOWNLOAD``, ``COLLABORATE``- View/Download+Upload+Edit: ``PUBLISH``, ``VIEW``, ``DOWNLOAD``, ``COLLABORATE``, ``EDIT``- Full controller: ``PUBLISH``, ``VIEW``, ``DOWNLOAD``, ``COLLABORATE``, ``EDIT``, ``CONTROL``- The six permission levels in Autodesk Construction Cloud correspond to one or more actions:- View Only: ``VIEW``, ``COLLABORATE``- View/Download: ``VIEW``, ``DOWNLOAD``, ``COLLABORATE``- View/Download+PublishMarkups: ``VIEW``, ``DOWNLOAD``, ``COLLABORATE``, ``PUBLISH_MARKUP``- View/Download+PublishMarkups+Upload: ``PUBLISH``, ``VIEW``, ``DOWNLOAD``, ``COLLABORATE``, ``PUBLISH_MARKUP``- View/Download+PublishMarkups+Upload+Edit: ``PUBLISH``, ``VIEW``, ``DOWNLOAD``, ``COLLABORATE``, ``PUBLISH_MARKUP``, ``EDIT``- Full controller: ``PUBLISH``, ``VIEW``, ``DOWNLOAD``, ``COLLABORATE``, ``PUBLISH_MARKUP``, ``EDIT``, ``CONTROL``See the `BIM 360 Help documentation <http://help.autodesk.com/view/BIM360D/ENU/?guid=GUID-2643FEEF-B48A-45A1-B354-797DAD628C37>`_ or the `ACC Files Help documentation <https://help.autodesk.com/view/BUILD/ENU/?guid=Folder_Permissions>`_ for more details about each permission level.Note that the full set of permissions assigned to the user, role, or company is a combination of ``actions`` and ``inheritActions``.
    actions: Optional[list[str]] = None
    # The ID of the user, role, or company. For example, this corresponds to the ``id``, ``roleId``, or ``companyId`` in the response for `GET /users/user_id </en/docs/bim360/v1/reference/http/admin-v1-projects-projectId-users-userId-GET/>`_.
    subject_id: Optional[UUID] = None
    # The type of subject.Possible values: ``USER``, ``COMPANY``, ``ROLE``
    subject_type: Optional[PermissionsBatchCreatePostResponse_results_subjectType] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> PermissionsBatchCreatePostResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: PermissionsBatchCreatePostResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return PermissionsBatchCreatePostResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .permissions_batch_create_post_response_results_subject_type import PermissionsBatchCreatePostResponse_results_subjectType

        from .permissions_batch_create_post_response_results_subject_type import PermissionsBatchCreatePostResponse_results_subjectType

        fields: dict[str, Callable[[Any], None]] = {
            "actions": lambda n : setattr(self, 'actions', n.get_collection_of_primitive_values(str)),
            "subjectId": lambda n : setattr(self, 'subject_id', n.get_uuid_value()),
            "subjectType": lambda n : setattr(self, 'subject_type', n.get_enum_value(PermissionsBatchCreatePostResponse_results_subjectType)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_primitive_values("actions", self.actions)
        writer.write_uuid_value("subjectId", self.subject_id)
        writer.write_enum_value("subjectType", self.subject_type)
    




from enum import Enum

class WithFormPatchRequestBody_status(str, Enum):
    Draft = "draft",
    Discarded = "discarded",
    Submitted = "submitted",
    Archived = "archived",
    In_review = "in_review",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class WithModelSetPatchRequestBody(Parsable):
    # The new description of the model set. Min length: 1 Max length: 1024.
    new_description: Optional[str] = None
    # The new name of the model set. This name must be unique within the specified container. Min length: 1 Max length: 64.
    new_name: Optional[str] = None
    # The current description of the model set. Min length: 1 Max length: 1024.
    old_description: Optional[str] = None
    # The current name of the model set. Min length: 1 Max length: 64.
    old_name: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithModelSetPatchRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithModelSetPatchRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithModelSetPatchRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "newDescription": lambda n : setattr(self, 'new_description', n.get_str_value()),
            "newName": lambda n : setattr(self, 'new_name', n.get_str_value()),
            "oldDescription": lambda n : setattr(self, 'old_description', n.get_str_value()),
            "oldName": lambda n : setattr(self, 'old_name', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("newDescription", self.new_description)
        writer.write_str_value("newName", self.new_name)
        writer.write_str_value("oldDescription", self.old_description)
        writer.write_str_value("oldName", self.old_name)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .with_step_get_response_permitted_actions import WithStepGetResponse_permittedActions
    from .with_step_get_response_status import WithStepGetResponse_status
    from .with_step_get_response_tasks import WithStepGetResponse_tasks

@dataclass
class WithStepGetResponse(Parsable):
    # The date and time when the step was completed, formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2025-01-20T12:00:00.198466Z``.
    completed_at: Optional[datetime.datetime] = None
    # The date and time when the step was created, formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2025-01-20T12:00:00.198466Z``.
    created_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user that created the step.
    created_by: Optional[str] = None
    # Specifies a dynamic due date. When the step starts, the due date is calculated based on this field.
    days_to_respond: Optional[float] = None
    # The due date of the step in the format ``YYYY-MM-DD`` (ISO 8601) in UTC. For example, ``2018-02-15``.
    due_date: Optional[str] = None
    # The internal, globally unique identifier (UUID) for the step.
    id: Optional[UUID] = None
    # The ID of the item associated with the step.
    item_id: Optional[UUID] = None
    # A list of actions that the user is allowed to perform on the step.
    permitted_actions: Optional[list[WithStepGetResponse_permittedActions]] = None
    # The date and time when the step transitioned to ``In Progress`` in the backend. This corresponds to the step being marked as ``Started`` in the UI, formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2025-01-20T12:00:00.198466Z``.
    started_at: Optional[datetime.datetime] = None
    # The current status of the step. Possible values: ``not-started``, ``in-progress``, ``completed``.
    status: Optional[WithStepGetResponse_status] = None
    # The number representing the order of the steps, where ``1`` is the first step.
    step_number: Optional[float] = None
    # The list of tasks associated with the step.
    tasks: Optional[list[WithStepGetResponse_tasks]] = None
    # The date and time when the step was last updated, formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2025-01-20T12:00:00.198466Z``.
    updated_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user that updated the step.
    updated_by: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithStepGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithStepGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithStepGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_step_get_response_permitted_actions import WithStepGetResponse_permittedActions
        from .with_step_get_response_status import WithStepGetResponse_status
        from .with_step_get_response_tasks import WithStepGetResponse_tasks

        from .with_step_get_response_permitted_actions import WithStepGetResponse_permittedActions
        from .with_step_get_response_status import WithStepGetResponse_status
        from .with_step_get_response_tasks import WithStepGetResponse_tasks

        fields: dict[str, Callable[[Any], None]] = {
            "completedAt": lambda n : setattr(self, 'completed_at', n.get_datetime_value()),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "daysToRespond": lambda n : setattr(self, 'days_to_respond', n.get_float_value()),
            "dueDate": lambda n : setattr(self, 'due_date', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "itemId": lambda n : setattr(self, 'item_id', n.get_uuid_value()),
            "permittedActions": lambda n : setattr(self, 'permitted_actions', n.get_collection_of_object_values(WithStepGetResponse_permittedActions)),
            "startedAt": lambda n : setattr(self, 'started_at', n.get_datetime_value()),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(WithStepGetResponse_status)),
            "stepNumber": lambda n : setattr(self, 'step_number', n.get_float_value()),
            "tasks": lambda n : setattr(self, 'tasks', n.get_collection_of_object_values(WithStepGetResponse_tasks)),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "updatedBy": lambda n : setattr(self, 'updated_by', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_datetime_value("completedAt", self.completed_at)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_float_value("daysToRespond", self.days_to_respond)
        writer.write_str_value("dueDate", self.due_date)
        writer.write_uuid_value("id", self.id)
        writer.write_uuid_value("itemId", self.item_id)
        writer.write_collection_of_object_values("permittedActions", self.permitted_actions)
        writer.write_datetime_value("startedAt", self.started_at)
        writer.write_enum_value("status", self.status)
        writer.write_float_value("stepNumber", self.step_number)
        writer.write_collection_of_object_values("tasks", self.tasks)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedBy", self.updated_by)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .admin.admin_request_builder import AdminRequestBuilder
    from .assets.assets_request_builder import AssetsRequestBuilder
    from .autospecs.autospecs_request_builder import AutospecsRequestBuilder
    from .files.files_request_builder import FilesRequestBuilder
    from .forms.forms_request_builder import FormsRequestBuilder
    from .index.index_request_builder import IndexRequestBuilder
    from .issues.issues_request_builder import IssuesRequestBuilder
    from .locations.locations_request_builder import LocationsRequestBuilder
    from .packages.packages_request_builder import PackagesRequestBuilder
    from .photos.photos_request_builder import PhotosRequestBuilder
    from .rcm.rcm_request_builder import RcmRequestBuilder
    from .reviews.reviews_request_builder import ReviewsRequestBuilder
    from .rfis.rfis_request_builder import RfisRequestBuilder
    from .sheets.sheets_request_builder import SheetsRequestBuilder
    from .submittals.submittals_request_builder import SubmittalsRequestBuilder
    from .takeoff.takeoff_request_builder import TakeoffRequestBuilder
    from .transmittals.transmittals_request_builder import TransmittalsRequestBuilder

class ConstructionRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new ConstructionRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction", path_parameters)
    
    @property
    def admin(self) -> AdminRequestBuilder:
        """
        The admin property
        """
        from .admin.admin_request_builder import AdminRequestBuilder

        return AdminRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def assets(self) -> AssetsRequestBuilder:
        """
        The assets property
        """
        from .assets.assets_request_builder import AssetsRequestBuilder

        return AssetsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def autospecs(self) -> AutospecsRequestBuilder:
        """
        The autospecs property
        """
        from .autospecs.autospecs_request_builder import AutospecsRequestBuilder

        return AutospecsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def files(self) -> FilesRequestBuilder:
        """
        The files property
        """
        from .files.files_request_builder import FilesRequestBuilder

        return FilesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def forms(self) -> FormsRequestBuilder:
        """
        The forms property
        """
        from .forms.forms_request_builder import FormsRequestBuilder

        return FormsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def index(self) -> IndexRequestBuilder:
        """
        The index property
        """
        from .index.index_request_builder import IndexRequestBuilder

        return IndexRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def issues(self) -> IssuesRequestBuilder:
        """
        The issues property
        """
        from .issues.issues_request_builder import IssuesRequestBuilder

        return IssuesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def locations(self) -> LocationsRequestBuilder:
        """
        The locations property
        """
        from .locations.locations_request_builder import LocationsRequestBuilder

        return LocationsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def packages(self) -> PackagesRequestBuilder:
        """
        The packages property
        """
        from .packages.packages_request_builder import PackagesRequestBuilder

        return PackagesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def photos(self) -> PhotosRequestBuilder:
        """
        The photos property
        """
        from .photos.photos_request_builder import PhotosRequestBuilder

        return PhotosRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def rcm(self) -> RcmRequestBuilder:
        """
        The rcm property
        """
        from .rcm.rcm_request_builder import RcmRequestBuilder

        return RcmRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def reviews(self) -> ReviewsRequestBuilder:
        """
        The reviews property
        """
        from .reviews.reviews_request_builder import ReviewsRequestBuilder

        return ReviewsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def rfis(self) -> RfisRequestBuilder:
        """
        The rfis property
        """
        from .rfis.rfis_request_builder import RfisRequestBuilder

        return RfisRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def sheets(self) -> SheetsRequestBuilder:
        """
        The sheets property
        """
        from .sheets.sheets_request_builder import SheetsRequestBuilder

        return SheetsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def submittals(self) -> SubmittalsRequestBuilder:
        """
        The submittals property
        """
        from .submittals.submittals_request_builder import SubmittalsRequestBuilder

        return SubmittalsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def takeoff(self) -> TakeoffRequestBuilder:
        """
        The takeoff property
        """
        from .takeoff.takeoff_request_builder import TakeoffRequestBuilder

        return TakeoffRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def transmittals(self) -> TransmittalsRequestBuilder:
        """
        The transmittals property
        """
        from .transmittals.transmittals_request_builder import TransmittalsRequestBuilder

        return TransmittalsRequestBuilder(self.request_adapter, self.path_parameters)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class RelationshipsSearchGetResponse_relationships_entities(Parsable):
    # The date and time the entity was created.
    created_on: Optional[datetime.datetime] = None
    # The domain to which the entity belongs.To learn more about domains and entities, see the `Relationship Service Field Guide </en/docs/bim360/v1/overview/field-guide/relationships>`_.Max length: 128
    domain: Optional[str] = None
    # The unique identifier of the entity.Max length: 512
    id: Optional[str] = None
    # The type of entity.Max length: 128
    type: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> RelationshipsSearchGetResponse_relationships_entities:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: RelationshipsSearchGetResponse_relationships_entities
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return RelationshipsSearchGetResponse_relationships_entities()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "createdOn": lambda n : setattr(self, 'created_on', n.get_datetime_value()),
            "domain": lambda n : setattr(self, 'domain', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "type": lambda n : setattr(self, 'type', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_datetime_value("createdOn", self.created_on)
        writer.write_str_value("domain", self.domain)
        writer.write_str_value("id", self.id)
        writer.write_str_value("type", self.type)
    




from enum import Enum

class PaymentItemsGetResponse_results_associationType(str, Enum):
    SCO = "SCO",
    OCO = "OCO",
    SOV = "SOV",
    CostItem = "CostItem",
    SubCostItem = "SubCostItem",
    MaterialsOnSite = "MaterialsOnSite",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID
from warnings import warn

if TYPE_CHECKING:
    from .custom_attributes_get_response import CustomAttributesGetResponse
    from .item.with_custom_attribute_item_request_builder import WithCustomAttributeItemRequestBuilder

class CustomAttributesRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/assets/v1/projects/{projectId}/categories/{categoryId}/custom-attributes
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new CustomAttributesRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/assets/v1/projects/{projectId}/categories/{categoryId}/custom-attributes{?includeInherited*}", path_parameters)
    
    def by_custom_attribute_id(self,custom_attribute_id: UUID) -> WithCustomAttributeItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.construction.assets.v1.projects.item.categories.item.customAttributes.item collection
        param custom_attribute_id: Asset custom attribute ID
        Returns: WithCustomAttributeItemRequestBuilder
        """
        if custom_attribute_id is None:
            raise TypeError("custom_attribute_id cannot be null.")
        from .item.with_custom_attribute_item_request_builder import WithCustomAttributeItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["customAttributeId"] = custom_attribute_id
        return WithCustomAttributeItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[CustomAttributesRequestBuilderGetQueryParameters]] = None) -> Optional[CustomAttributesGetResponse]:
        """
        GET categories/:categoryId/custom-attributes
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[CustomAttributesGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .custom_attributes_get_response import CustomAttributesGetResponse

        return await self.request_adapter.send_async(request_info, CustomAttributesGetResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[CustomAttributesRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        GET categories/:categoryId/custom-attributes
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def with_url(self,raw_url: str) -> CustomAttributesRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: CustomAttributesRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return CustomAttributesRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class CustomAttributesRequestBuilderGetQueryParameters():
        """
        GET categories/:categoryId/custom-attributes
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "include_inherited":
                return "includeInherited"
            return original_name
        
        # Specifies whether or not to return custom attributes that were inherited from the specified category'sparent category. If ``true``, then it returns inherited custom attributes. If ``false``, then it returns only customattributes explicitly assigned to the specified category. Default is ``false``.
        include_inherited: Optional[bool] = None

    
    @dataclass
    class CustomAttributesRequestBuilderGetRequestConfiguration(RequestConfiguration[CustomAttributesRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class WithQueryGetResponse_errors_errors(Parsable):
    # A more detailed, human readable description of the error, assuming that this message is not localized and is therefore EN-US. UI consumers can use the error.type value to provide a localized version of this error for presentation.
    detail: Optional[str] = None
    # The field which failed validation.
    field: Optional[str] = None
    # A short title for the error.
    title: Optional[str] = None
    # The error code.
    type: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithQueryGetResponse_errors_errors:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithQueryGetResponse_errors_errors
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithQueryGetResponse_errors_errors()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "detail": lambda n : setattr(self, 'detail', n.get_str_value()),
            "field": lambda n : setattr(self, 'field', n.get_str_value()),
            "title": lambda n : setattr(self, 'title', n.get_str_value()),
            "type": lambda n : setattr(self, 'type', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("detail", self.detail)
        writer.write_str_value("field", self.field)
        writer.write_str_value("title", self.title)
        writer.write_str_value("type", self.type)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .issues.issues_request_builder import IssuesRequestBuilder
    from .jobs.jobs_request_builder import JobsRequestBuilder
    from .screenshots.screenshots_request_builder import ScreenshotsRequestBuilder
    from .versions.versions_request_builder import VersionsRequestBuilder
    from .versions_disable.versions_disable_request_builder import VersionsDisableRequestBuilder
    from .versions_enable.versions_enable_request_builder import VersionsEnableRequestBuilder
    from .views.views_request_builder import ViewsRequestBuilder
    from .views_lineages.views_lineages_request_builder import ViewsLineagesRequestBuilder
    from .with_model_set_get_response import WithModelSetGetResponse
    from .with_model_set_patch_request_body import WithModelSetPatchRequestBody
    from .with_model_set_patch_response import WithModelSetPatchResponse

class WithModelSetItemRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /bim360/modelset/v3/containers/{containerId}/modelsets/{modelSetId}
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new WithModelSetItemRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/bim360/modelset/v3/containers/{containerId}/modelsets/{modelSetId}", path_parameters)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[WithModelSetGetResponse]:
        """
        Returns the requested model set object.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[WithModelSetGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .with_model_set_get_response import WithModelSetGetResponse

        return await self.request_adapter.send_async(request_info, WithModelSetGetResponse, None)
    
    async def patch(self,body: WithModelSetPatchRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[WithModelSetPatchResponse]:
        """
        If no property changes are supplied, an error is returned.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[WithModelSetPatchResponse]
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_patch_request_information(
            body, request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .with_model_set_patch_response import WithModelSetPatchResponse

        return await self.request_adapter.send_async(request_info, WithModelSetPatchResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        Returns the requested model set object.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_patch_request_information(self,body: WithModelSetPatchRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        If no property changes are supplied, an error is returned.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.PATCH, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> WithModelSetItemRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: WithModelSetItemRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return WithModelSetItemRequestBuilder(self.request_adapter, raw_url)
    
    @property
    def issues(self) -> IssuesRequestBuilder:
        """
        The issues property
        """
        from .issues.issues_request_builder import IssuesRequestBuilder

        return IssuesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def jobs(self) -> JobsRequestBuilder:
        """
        The jobs property
        """
        from .jobs.jobs_request_builder import JobsRequestBuilder

        return JobsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def screenshots(self) -> ScreenshotsRequestBuilder:
        """
        The screenshots property
        """
        from .screenshots.screenshots_request_builder import ScreenshotsRequestBuilder

        return ScreenshotsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def versions(self) -> VersionsRequestBuilder:
        """
        The versions property
        """
        from .versions.versions_request_builder import VersionsRequestBuilder

        return VersionsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def versions_disable(self) -> VersionsDisableRequestBuilder:
        """
        The versionsDisable property
        """
        from .versions_disable.versions_disable_request_builder import VersionsDisableRequestBuilder

        return VersionsDisableRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def versions_enable(self) -> VersionsEnableRequestBuilder:
        """
        The versionsEnable property
        """
        from .versions_enable.versions_enable_request_builder import VersionsEnableRequestBuilder

        return VersionsEnableRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def views(self) -> ViewsRequestBuilder:
        """
        The views property
        """
        from .views.views_request_builder import ViewsRequestBuilder

        return ViewsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def views_lineages(self) -> ViewsLineagesRequestBuilder:
        """
        The viewsLineages property
        """
        from .views_lineages.views_lineages_request_builder import ViewsLineagesRequestBuilder

        return ViewsLineagesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @dataclass
    class WithModelSetItemRequestBuilderGetRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class WithModelSetItemRequestBuilderPatchRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .uploads_post_request_body_files import UploadsPostRequestBody_files

@dataclass
class UploadsPostRequestBody(Parsable):
    # The list of files to be processed by the upload.
    files: Optional[list[UploadsPostRequestBody_files]] = None
    # The ID of the version set you want to upload the sheets to.To create a new version set, call `POST version-sets </en/docs/acc/v1/reference/http/sheets-version-sets-POST/>`_.To find the ID of an existing version set, call `GET version-sets </en/docs/acc/v1/reference/http/sheets-version-sets-GET/>`_.
    version_set_id: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> UploadsPostRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: UploadsPostRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return UploadsPostRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .uploads_post_request_body_files import UploadsPostRequestBody_files

        from .uploads_post_request_body_files import UploadsPostRequestBody_files

        fields: dict[str, Callable[[Any], None]] = {
            "files": lambda n : setattr(self, 'files', n.get_collection_of_object_values(UploadsPostRequestBody_files)),
            "versionSetId": lambda n : setattr(self, 'version_set_id', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("files", self.files)
        writer.write_str_value("versionSetId", self.version_set_id)
    




from enum import Enum

class GetFilterAssociationTypeQueryParameterType(str, Enum):
    Contract = "Contract",
    MainContract = "MainContract",
    FilterAssociationTypeContract = "filter[associationType]=Contract",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .me_get_response_permitted_actions import MeGetResponse_permittedActions

@dataclass
class MeGetResponse(Parsable):
    # The AutodeskId for the current user.
    id: Optional[str] = None
    # A list of actions that the user is allowed to perform
    permitted_actions: Optional[list[MeGetResponse_permittedActions]] = None
    # The roles assigned to the user in Submittals. Possible values: ``1`` - ``Manager``, ``2`` - ``User``, ``4`` - ``Admin``
    roles: Optional[list[str]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> MeGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: MeGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return MeGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .me_get_response_permitted_actions import MeGetResponse_permittedActions

        from .me_get_response_permitted_actions import MeGetResponse_permittedActions

        fields: dict[str, Callable[[Any], None]] = {
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "permittedActions": lambda n : setattr(self, 'permitted_actions', n.get_collection_of_object_values(MeGetResponse_permittedActions)),
            "roles": lambda n : setattr(self, 'roles', n.get_collection_of_primitive_values(str)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("id", self.id)
        writer.write_collection_of_object_values("permittedActions", self.permitted_actions)
        writer.write_collection_of_primitive_values("roles", self.roles)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID
from warnings import warn

if TYPE_CHECKING:
    from .get_filter_type_query_parameter_type import GetFilterTypeQueryParameterType
    from .item.with_sub_cost_items_item_request_builder import WithSubCostItemsItemRequestBuilder
    from .sub_cost_items_get_response import SubCostItemsGetResponse
    from .sub_cost_items_post_request_body import SubCostItemsPostRequestBody
    from .sub_cost_items_post_response import SubCostItemsPostResponse

class SubCostItemsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /cost/v1/containers/{containerId}/cost-items/{costItemId}/sub-cost-items
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new SubCostItemsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/cost/v1/containers/{containerId}/cost-items/{costItemId}/sub-cost-items{?filter%5Btype%5D*,limit*,offset*,sort*}", path_parameters)
    
    def by_sub_cost_items_id(self,sub_cost_items_id: UUID) -> WithSubCostItemsItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.cost.v1.containers.item.costItems.item.subCostItems.item collection
        param sub_cost_items_id: The ID of the sub cost item to be updated. To find the sub cost item ID, call `GET sub-cost-items </en/docs/bim360/v1/reference/http/cost-sub-cost-items-GET/>`_.
        Returns: WithSubCostItemsItemRequestBuilder
        """
        if sub_cost_items_id is None:
            raise TypeError("sub_cost_items_id cannot be null.")
        from .item.with_sub_cost_items_item_request_builder import WithSubCostItemsItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["subCostItemsId"] = sub_cost_items_id
        return WithSubCostItemsItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[SubCostItemsRequestBuilderGetQueryParameters]] = None) -> Optional[SubCostItemsGetResponse]:
        """
        Retrieves sub cost items associated with a specific cost item in a project.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[SubCostItemsGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .sub_cost_items_get_response import SubCostItemsGetResponse

        return await self.request_adapter.send_async(request_info, SubCostItemsGetResponse, None)
    
    async def post(self,body: SubCostItemsPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[SubCostItemsPostResponse]:
        """
        Creates a sub cost item within a specific cost item in a project. Sub cost items store detailed cost breakdowns and can be used for tracking and categorization.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[SubCostItemsPostResponse]
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_post_request_information(
            body, request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .sub_cost_items_post_response import SubCostItemsPostResponse

        return await self.request_adapter.send_async(request_info, SubCostItemsPostResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[SubCostItemsRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Retrieves sub cost items associated with a specific cost item in a project.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_post_request_information(self,body: SubCostItemsPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        Creates a sub cost item within a specific cost item in a project. Sub cost items store detailed cost breakdowns and can be used for tracking and categorization.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.POST, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> SubCostItemsRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: SubCostItemsRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return SubCostItemsRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class SubCostItemsRequestBuilderGetQueryParameters():
        """
        Retrieves sub cost items associated with a specific cost item in a project.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "filtertype":
                return "filter%5Btype%5D"
            if original_name == "limit":
                return "limit"
            if original_name == "offset":
                return "offset"
            if original_name == "sort":
                return "sort"
            return original_name
        
        # The type of the value to break down.Possible values: ``estimated``, ``proposed``, ``submitted``, ``approved``, ``committed``
        filtertype: Optional[GetFilterTypeQueryParameterType] = None

        # The maximum number of records returned per page. Default: ``100``. A page may contain fewer records than the limit if there are fewer matching items or if it is the last page of results.
        limit: Optional[int] = None

        # The number of records to skip before returning results. Used together with ``limit`` to paginate through results, where ``offset`` specifies the starting point and ``limit`` specifies the number of records to return.
        offset: Optional[int] = None

        # Defines the sort order for the results. Each attribute can be sorted in ``asc`` (default) or ``desc`` order. For example, ``sort=name desc`` sorts the results by name in descending order.
        sort: Optional[str] = None

    
    @dataclass
    class SubCostItemsRequestBuilderGetRequestConfiguration(RequestConfiguration[SubCostItemsRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class SubCostItemsRequestBuilderPostRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .property_values_batch_update_post_request_body_association_type import PropertyValuesBatchUpdatePostRequestBody_associationType

@dataclass
class PropertyValuesBatchUpdatePostRequestBody(Parsable):
    # The object ID of the item associated with the actions, such as a budget, contract, or cost item.
    association_id: Optional[UUID] = None
    # The type of item to which it is associated.Possible values: ``Budget``, ``Contract``, ``ScheduleOfValue``, ``FormInstance``, ``CostItem``, ``Payment``, ``MainContract``, ``BudgetPayment``, ``Expense``, ``CostPayment``, ``ExpenseItem``, ``PaymentItem``, ``OCO``, ``RCO``, ``SCO``, ``PCO``, ``RFQ``, ``DistributionItem``, ``BudgetTransfer``, ``Fee``
    association_type: Optional[PropertyValuesBatchUpdatePostRequestBody_associationType] = None
    # ID of the custom attribute definition.
    property_definition_id: Optional[UUID] = None
    # Value of the custom attribute associated to an item.
    value: Optional[float] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> PropertyValuesBatchUpdatePostRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: PropertyValuesBatchUpdatePostRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return PropertyValuesBatchUpdatePostRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .property_values_batch_update_post_request_body_association_type import PropertyValuesBatchUpdatePostRequestBody_associationType

        from .property_values_batch_update_post_request_body_association_type import PropertyValuesBatchUpdatePostRequestBody_associationType

        fields: dict[str, Callable[[Any], None]] = {
            "associationId": lambda n : setattr(self, 'association_id', n.get_uuid_value()),
            "associationType": lambda n : setattr(self, 'association_type', n.get_enum_value(PropertyValuesBatchUpdatePostRequestBody_associationType)),
            "propertyDefinitionId": lambda n : setattr(self, 'property_definition_id', n.get_uuid_value()),
            "value": lambda n : setattr(self, 'value', n.get_float_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_uuid_value("associationId", self.association_id)
        writer.write_enum_value("associationType", self.association_type)
        writer.write_uuid_value("propertyDefinitionId", self.property_definition_id)
        writer.write_float_value("value", self.value)
    




from enum import Enum

class GetFilterTextMatchQueryParameterType(str, Enum):
    FilterName = "filter[name]",
    FilterJobNumber = "filter[jobNumber]",
    FilterCompanyName = "filter[companyName]",
    Contains = "contains",
    StartsWith = "startsWith",
    EndsWith = "endsWith",
    Equals = "equals",




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .payment_items_get_response_results_association_type import PaymentItemsGetResponse_results_associationType

@dataclass
class PaymentItemsGetResponse_results(Parsable):
    # The advance amount of the work completed in this period.
    advance_amount: Optional[float] = None
    # The advance percent of the work completed in this period.
    advance_percent: Optional[float] = None
    # The aggregate type to budget Payment application.Possible values: ``workCompleted``, ``workCompletedPercentage``
    aggregate_by: Optional[str] = None
    # The amount of the work completed in this period.
    amount: Optional[float] = None
    # The ID of the object that the payment item is generated from.
    association_id: Optional[UUID] = None
    # The object type of the payment item is generated from.Possible values: ``SCO``, ``OCO``, ``SOV``, ``CostItem``, ``SubCostItem``, ``MaterialsOnSite``
    association_type: Optional[PaymentItemsGetResponse_results_associationType] = None
    # The budget code for the payment item (unformatted string).
    budget_code: Optional[str] = None
    # The budget code for the payment item (formatted with separators).
    budget_formatted_code: Optional[str] = None
    # The ID of the Budget to which the payment item belongs.
    budget_id: Optional[UUID] = None
    # Not relevant
    budget_transfers: Optional[float] = None
    # Not relevant
    budget_transfers_quantity: Optional[float] = None
    # Not relevant
    budget_transfers_unit_price: Optional[float] = None
    # Indicate if this payment item can be deleted.
    can_delete: Optional[bool] = None
    # The user who made the change.
    changed_by: Optional[str] = None
    # The claimed amount of the work completed in this period.
    claimed_amount: Optional[float] = None
    # The claimed quantity of the work completed in this period.
    claimed_quantity: Optional[float] = None
    # The claimed unit price of the work completed in this period.
    claimed_unit_price: Optional[float] = None
    # The retained amount of total work completed released in this period.
    completed_work_released: Optional[float] = None
    # Not relevant
    completed_work_retained: Optional[float] = None
    # Not relevant
    completed_work_retained_percent: Optional[float] = None
    # The amount of total work completed retained (pre-release) in this application. This parameter will be deprecated.
    completed_work_retention: Optional[float] = None
    # The percentage of total work completed retained in this application.
    completed_work_retention_percent: Optional[float] = None
    # The date and time that the item was created, in ISO 8601 format.
    created_at: Optional[datetime.datetime] = None
    # The user who created the payment item. This is the ID of a user managed by BIM 360 Admin.
    creator_id: Optional[str] = None
    # Not relevant
    current_completed_work_retention: Optional[float] = None
    # Not relevant
    current_completed_work_retention_foreign_currency: Optional[float] = None
    # Not relevant
    current_completed_work_retention_percent: Optional[float] = None
    # Not relevant
    current_materials_retention: Optional[float] = None
    # Not relevant
    current_materials_retention_foreign_currency: Optional[float] = None
    # Not relevant
    current_materials_retention_percent: Optional[float] = None
    # Not relevant
    current_released: Optional[float] = None
    # Not relevant
    current_retention: Optional[float] = None
    # A description of the payment item.
    description: Optional[str] = None
    # Exchange rate. Default value is ``1``. If multi-currency is not enabled, it will also be ``1``.
    exchange_rate: Optional[float] = None
    # The identifier assigned to an item in its original external ERP system. Use this ID to track and look up data within the integrated system. Note that this value comes from the item's ID in the external system.Max length: 255
    external_id: Optional[str] = None
    # A message generated by the external ERP system that explains the sync status of the integration. For example, common values include ``success`` or ``fail`` to indicate the result of the integration operation.Max length: 255
    external_message: Optional[str] = None
    # The name of the external ERP system integrated with Cost Management. Use this name to identify and search for data within the integrated system.Max length: 255
    external_system: Optional[str] = None
    # Indicate if this payment item has comment.
    has_comment: Optional[bool] = None
    # Unique identifier of the payment.
    id: Optional[UUID] = None
    # The state of the item during the integration with the external ERP system (such as SignNow). An item can be a ``budget``, ``contract``, ``main contract``, ``main contract item``, ``cost item``, ``expense``, ``expense item``, ``change order``, or ``schedule of value``. For more details, see `Integrate with External System </en/docs/acc/v1/tutorials/cost/integrate-with-external-system/>`_ tutorial.Possible values:``locked``: the item is currently locked within the ERP system, preventing modifications until unlocked. To unlock and modify the item, use the relevant PATCH endpoint to set ``integrationState`` to ``null``. For example, for a budget, call `PATCH budgets <en/docs/bim360/v1/reference/http/cost-budgets-budgetId-PATCH/>`_. For a contract, call `PATCH contracts <en/docs/bim360/v1/reference/http/cost-contracts-contractId-PATCH/>`_. For more details, see the `Help documentation <https://help.autodesk.com/view/BUILD/ENU/?guid=Integrated_and_Locked>`_.``integrated``: the item has been successfully added to the ERP system.``failed``: the item encountered an error during the integration process and was not successfully added to the ERP system. For example, if a user tries to integrate ``contracts`` from an ERP system and the updates fail, the ``integrationState`` can be set to ``failed``. Retry the sync process or analyze the issue if it continues to fail.``null``: The item has not been integrated with the ERP system. This is default value.For more information regarding integrations within the Cost Management system, see `Integrations in Cost Management <https://help.autodesk.com/view/BUILD/ENU/?guid=Cost_Integrations>`_.
    integration_state: Optional[str] = None
    # The date and time that the item's integration status was last changed.
    integration_state_changed_at: Optional[str] = None
    # The user who last changed the integration status. This is the ID of a user managed by the BIM 360/ACC Admin.
    integration_state_changed_by: Optional[str] = None
    # Indicate if this payment item should be exposed to owner. This is only for budget payment.
    is_private: Optional[bool] = None
    # The last user who made the review.
    last_reviewed_by: Optional[str] = None
    # The date and time when the item was last synchronized with the external ERP system. This value is updated by the external system and is in ISO 8601 format.
    last_sync_time: Optional[datetime.datetime] = None
    # Not relevant
    materials: Optional[float] = None
    # The amount of the materials billed in this period.
    materials_billed: Optional[float] = None
    # The quantity of the materials billed in this period.
    materials_billed_quantity: Optional[float] = None
    # The unit of the materials billed in this period.
    materials_billed_unit: Optional[str] = None
    # The unit price of the materials billed in this period.
    materials_billed_unit_price: Optional[float] = None
    # The amount of the materials stored in this application.
    materials_on_store: Optional[float] = None
    # The quantity of the materials stored in this application.
    materials_on_store_quantity: Optional[float] = None
    # The retained amount of the materials (stored or billed) released in this period. This parameter is deprecated, use materialsReleased.
    materials_on_store_released: Optional[float] = None
    # The percentage of the materials retained. This parameter is deprecated, use materialsRetainedPercent.
    materials_on_store_retention_percent: Optional[float] = None
    # The unit of the materials stored in this application.
    materials_on_store_unit: Optional[str] = None
    # The unit price of the materials stored in this application.
    materials_on_store_unit_price: Optional[float] = None
    # The retained amount of the materials (stored or billed) released in this period.
    materials_released: Optional[float] = None
    # The amount of the materials (stored or billed) retained in this application.
    materials_retained: Optional[float] = None
    # Not relevant
    materials_retained_percent: Optional[float] = None
    # The amount of the materials (stored or billed) retained (pre-release) in this application. This parameter will be deprecated, use materialsRetained.
    materials_retention: Optional[float] = None
    # The percentage of the materials (stored or billed) retained (pre-release) in this application. This parameter will be deprecated, use materialsRetainedPercent.
    materials_retention_percent: Optional[float] = None
    # The name of the payment item.Max length: 1024
    name: Optional[str] = None
    # The net amount to be paid in this period, including work completed, materials stored or billed, retention and release.
    net_amount: Optional[float] = None
    # The net amount to be paid in this period, in foreign currency.
    net_amount_foreign_currency: Optional[float] = None
    # The original amount of the payment item. Derived from the associated SOV or the main contract item.
    original_amount: Optional[float] = None
    # If this payment item came from Schedule of Value, the value is equal to Schedule of Value exchange rate; If this payment item came from SCO, the value is equal to SCO exchange rate. Default value is ``1``. If multi-currency is not enabled, it will also be ``1``.
    original_exchange_rate: Optional[float] = None
    # The original quantity of the payment item. Derived from the associated SOV or the main contract item.
    original_quantity: Optional[float] = None
    # The original unit price of the payment item. Derived from the associated SOV or the main contract item.
    original_unit_price: Optional[float] = None
    # The ID of the parent item for sub items.
    parent_id: Optional[UUID] = None
    # The ID of the payment the items belong to.
    payment_id: Optional[UUID] = None
    # The position of the payment item in the payment item siblings.
    position: Optional[int] = None
    # The advance amount of total work completed prior to this period.
    previous_advance_amount: Optional[float] = None
    # The advance amount of total work completed prior to this period, in foreign currency.
    previous_advance_amount_foreign_currency: Optional[float] = None
    # The amount of total work completed prior to this period.
    previous_amount: Optional[float] = None
    # The amount of total work completed prior to this period, in foreign currency.
    previous_amount_foreign_currency: Optional[float] = None
    # The claimed amount of the work completed prior to this period.
    previous_claimed_amount: Optional[float] = None
    # The claimed amount of the work completed prior to this period, in foreign currency.
    previous_claimed_amount_foreign_currency: Optional[float] = None
    # The claimed quantity of total work completed prior to this period.
    previous_claimed_quantity: Optional[float] = None
    # The claimed unit price of the work completed prior to this period.
    previous_claimed_unit_price: Optional[float] = None
    # Not relevant
    previous_completed_work_retained: Optional[float] = None
    # Not relevant
    previous_completed_work_retained_foreign_currency: Optional[float] = None
    # Exchange rate from previous payment application. Default value is ``1``, if multi-currency is not enabled, it will also be ``1``.
    previous_exchange_rate: Optional[float] = None
    # Not relevant
    previous_materials: Optional[float] = None
    # The amount of total materials billed prior to this period.
    previous_materials_billed: Optional[float] = None
    # The amount of total materials billed prior to this period, in foreign currency.
    previous_materials_billed_foreign_currency: Optional[float] = None
    # The quantity of total materials billed prior to this period.
    previous_materials_billed_quantity: Optional[float] = None
    # The unit of total materials billed prior to this period.
    previous_materials_billed_unit: Optional[str] = None
    # The unit price of total materials billed prior to this period.
    previous_materials_billed_unit_price: Optional[float] = None
    # The amount of total materials stored prior to this period.
    previous_materials_on_store: Optional[float] = None
    # Not relevant
    previous_materials_retained: Optional[float] = None
    # Not relevant
    previous_materials_retained_foreign_currency: Optional[float] = None
    # Not relevant
    previous_net_amount: Optional[float] = None
    # The quantity of total work completed prior to this period.
    previous_quantity: Optional[float] = None
    # The recoupment amount of the work completed prior to this period.
    previous_recoupment_amount: Optional[float] = None
    # The recoupment amount of the work completed prior to this period, in foreign currency.
    previous_recoupment_amount_foreign_currency: Optional[float] = None
    # Not relevant
    previous_retained: Optional[float] = None
    # The unit price of total work completed prior to this period.
    previous_unit_price: Optional[float] = None
    # The quantity of the work completed in this period.
    quantity: Optional[float] = None
    # netAmountForeignCurrency / originalExchangeRate - netAmount
    realized_gain_or_loss: Optional[float] = None
    # The recoupment amount of the work completed in this period.
    recoupment_amount: Optional[float] = None
    # The recoupment percentage of the work completed in this period.
    recoupment_percent_of_completed_work: Optional[float] = None
    # Not relevant
    retainage: Optional[float] = None
    # The review status the payment item.Possible values: ``accepted``, ``rejected``, ``semi-rejected``, ``null``
    status: Optional[str] = None
    # Not relevant
    total_retained: Optional[float] = None
    # Not relevant
    total_retained_percent: Optional[float] = None
    # The percentage of total work completed and materials retained (pre-release).
    total_retention_percent: Optional[float] = None
    # The unit of the work completed in this period.
    unit: Optional[str] = None
    # The unit price of the work completed in this period.
    unit_price: Optional[float] = None
    # The date and time that the item was last updated, in ISO 8601 format.
    updated_at: Optional[datetime.datetime] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> PaymentItemsGetResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: PaymentItemsGetResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return PaymentItemsGetResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .payment_items_get_response_results_association_type import PaymentItemsGetResponse_results_associationType

        from .payment_items_get_response_results_association_type import PaymentItemsGetResponse_results_associationType

        fields: dict[str, Callable[[Any], None]] = {
            "advanceAmount": lambda n : setattr(self, 'advance_amount', n.get_float_value()),
            "advancePercent": lambda n : setattr(self, 'advance_percent', n.get_float_value()),
            "aggregateBy": lambda n : setattr(self, 'aggregate_by', n.get_str_value()),
            "amount": lambda n : setattr(self, 'amount', n.get_float_value()),
            "associationId": lambda n : setattr(self, 'association_id', n.get_uuid_value()),
            "associationType": lambda n : setattr(self, 'association_type', n.get_enum_value(PaymentItemsGetResponse_results_associationType)),
            "budgetCode": lambda n : setattr(self, 'budget_code', n.get_str_value()),
            "budgetFormattedCode": lambda n : setattr(self, 'budget_formatted_code', n.get_str_value()),
            "budgetId": lambda n : setattr(self, 'budget_id', n.get_uuid_value()),
            "budgetTransfers": lambda n : setattr(self, 'budget_transfers', n.get_float_value()),
            "budgetTransfersQuantity": lambda n : setattr(self, 'budget_transfers_quantity', n.get_float_value()),
            "budgetTransfersUnitPrice": lambda n : setattr(self, 'budget_transfers_unit_price', n.get_float_value()),
            "canDelete": lambda n : setattr(self, 'can_delete', n.get_bool_value()),
            "changedBy": lambda n : setattr(self, 'changed_by', n.get_str_value()),
            "claimedAmount": lambda n : setattr(self, 'claimed_amount', n.get_float_value()),
            "claimedQuantity": lambda n : setattr(self, 'claimed_quantity', n.get_float_value()),
            "claimedUnitPrice": lambda n : setattr(self, 'claimed_unit_price', n.get_float_value()),
            "completedWorkReleased": lambda n : setattr(self, 'completed_work_released', n.get_float_value()),
            "completedWorkRetained": lambda n : setattr(self, 'completed_work_retained', n.get_float_value()),
            "completedWorkRetainedPercent": lambda n : setattr(self, 'completed_work_retained_percent', n.get_float_value()),
            "completedWorkRetention": lambda n : setattr(self, 'completed_work_retention', n.get_float_value()),
            "completedWorkRetentionPercent": lambda n : setattr(self, 'completed_work_retention_percent', n.get_float_value()),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "creatorId": lambda n : setattr(self, 'creator_id', n.get_str_value()),
            "currentCompletedWorkRetention": lambda n : setattr(self, 'current_completed_work_retention', n.get_float_value()),
            "currentCompletedWorkRetentionForeignCurrency": lambda n : setattr(self, 'current_completed_work_retention_foreign_currency', n.get_float_value()),
            "currentCompletedWorkRetentionPercent": lambda n : setattr(self, 'current_completed_work_retention_percent', n.get_float_value()),
            "currentMaterialsRetention": lambda n : setattr(self, 'current_materials_retention', n.get_float_value()),
            "currentMaterialsRetentionForeignCurrency": lambda n : setattr(self, 'current_materials_retention_foreign_currency', n.get_float_value()),
            "currentMaterialsRetentionPercent": lambda n : setattr(self, 'current_materials_retention_percent', n.get_float_value()),
            "currentReleased": lambda n : setattr(self, 'current_released', n.get_float_value()),
            "currentRetention": lambda n : setattr(self, 'current_retention', n.get_float_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "exchangeRate": lambda n : setattr(self, 'exchange_rate', n.get_float_value()),
            "externalId": lambda n : setattr(self, 'external_id', n.get_str_value()),
            "externalMessage": lambda n : setattr(self, 'external_message', n.get_str_value()),
            "externalSystem": lambda n : setattr(self, 'external_system', n.get_str_value()),
            "hasComment": lambda n : setattr(self, 'has_comment', n.get_bool_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "integrationState": lambda n : setattr(self, 'integration_state', n.get_str_value()),
            "integrationStateChangedAt": lambda n : setattr(self, 'integration_state_changed_at', n.get_str_value()),
            "integrationStateChangedBy": lambda n : setattr(self, 'integration_state_changed_by', n.get_str_value()),
            "isPrivate": lambda n : setattr(self, 'is_private', n.get_bool_value()),
            "lastReviewedBy": lambda n : setattr(self, 'last_reviewed_by', n.get_str_value()),
            "lastSyncTime": lambda n : setattr(self, 'last_sync_time', n.get_datetime_value()),
            "materials": lambda n : setattr(self, 'materials', n.get_float_value()),
            "materialsBilled": lambda n : setattr(self, 'materials_billed', n.get_float_value()),
            "materialsBilledQuantity": lambda n : setattr(self, 'materials_billed_quantity', n.get_float_value()),
            "materialsBilledUnit": lambda n : setattr(self, 'materials_billed_unit', n.get_str_value()),
            "materialsBilledUnitPrice": lambda n : setattr(self, 'materials_billed_unit_price', n.get_float_value()),
            "materialsOnStore": lambda n : setattr(self, 'materials_on_store', n.get_float_value()),
            "materialsOnStoreQuantity": lambda n : setattr(self, 'materials_on_store_quantity', n.get_float_value()),
            "materialsOnStoreReleased": lambda n : setattr(self, 'materials_on_store_released', n.get_float_value()),
            "materialsOnStoreRetentionPercent": lambda n : setattr(self, 'materials_on_store_retention_percent', n.get_float_value()),
            "materialsOnStoreUnit": lambda n : setattr(self, 'materials_on_store_unit', n.get_str_value()),
            "materialsOnStoreUnitPrice": lambda n : setattr(self, 'materials_on_store_unit_price', n.get_float_value()),
            "materialsReleased": lambda n : setattr(self, 'materials_released', n.get_float_value()),
            "materialsRetained": lambda n : setattr(self, 'materials_retained', n.get_float_value()),
            "materialsRetainedPercent": lambda n : setattr(self, 'materials_retained_percent', n.get_float_value()),
            "materialsRetention": lambda n : setattr(self, 'materials_retention', n.get_float_value()),
            "materialsRetentionPercent": lambda n : setattr(self, 'materials_retention_percent', n.get_float_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "netAmount": lambda n : setattr(self, 'net_amount', n.get_float_value()),
            "netAmountForeignCurrency": lambda n : setattr(self, 'net_amount_foreign_currency', n.get_float_value()),
            "originalAmount": lambda n : setattr(self, 'original_amount', n.get_float_value()),
            "originalExchangeRate": lambda n : setattr(self, 'original_exchange_rate', n.get_float_value()),
            "originalQuantity": lambda n : setattr(self, 'original_quantity', n.get_float_value()),
            "originalUnitPrice": lambda n : setattr(self, 'original_unit_price', n.get_float_value()),
            "parentId": lambda n : setattr(self, 'parent_id', n.get_uuid_value()),
            "paymentId": lambda n : setattr(self, 'payment_id', n.get_uuid_value()),
            "position": lambda n : setattr(self, 'position', n.get_int_value()),
            "previousAdvanceAmount": lambda n : setattr(self, 'previous_advance_amount', n.get_float_value()),
            "previousAdvanceAmountForeignCurrency": lambda n : setattr(self, 'previous_advance_amount_foreign_currency', n.get_float_value()),
            "previousAmount": lambda n : setattr(self, 'previous_amount', n.get_float_value()),
            "previousAmountForeignCurrency": lambda n : setattr(self, 'previous_amount_foreign_currency', n.get_float_value()),
            "previousClaimedAmount": lambda n : setattr(self, 'previous_claimed_amount', n.get_float_value()),
            "previousClaimedAmountForeignCurrency": lambda n : setattr(self, 'previous_claimed_amount_foreign_currency', n.get_float_value()),
            "previousClaimedQuantity": lambda n : setattr(self, 'previous_claimed_quantity', n.get_float_value()),
            "previousClaimedUnitPrice": lambda n : setattr(self, 'previous_claimed_unit_price', n.get_float_value()),
            "previousCompletedWorkRetained": lambda n : setattr(self, 'previous_completed_work_retained', n.get_float_value()),
            "previousCompletedWorkRetainedForeignCurrency": lambda n : setattr(self, 'previous_completed_work_retained_foreign_currency', n.get_float_value()),
            "previousExchangeRate": lambda n : setattr(self, 'previous_exchange_rate', n.get_float_value()),
            "previousMaterials": lambda n : setattr(self, 'previous_materials', n.get_float_value()),
            "previousMaterialsBilled": lambda n : setattr(self, 'previous_materials_billed', n.get_float_value()),
            "previousMaterialsBilledForeignCurrency": lambda n : setattr(self, 'previous_materials_billed_foreign_currency', n.get_float_value()),
            "previousMaterialsBilledQuantity": lambda n : setattr(self, 'previous_materials_billed_quantity', n.get_float_value()),
            "previousMaterialsBilledUnit": lambda n : setattr(self, 'previous_materials_billed_unit', n.get_str_value()),
            "previousMaterialsBilledUnitPrice": lambda n : setattr(self, 'previous_materials_billed_unit_price', n.get_float_value()),
            "previousMaterialsOnStore": lambda n : setattr(self, 'previous_materials_on_store', n.get_float_value()),
            "previousMaterialsRetained": lambda n : setattr(self, 'previous_materials_retained', n.get_float_value()),
            "previousMaterialsRetainedForeignCurrency": lambda n : setattr(self, 'previous_materials_retained_foreign_currency', n.get_float_value()),
            "previousNetAmount": lambda n : setattr(self, 'previous_net_amount', n.get_float_value()),
            "previousQuantity": lambda n : setattr(self, 'previous_quantity', n.get_float_value()),
            "previousRecoupmentAmount": lambda n : setattr(self, 'previous_recoupment_amount', n.get_float_value()),
            "previousRecoupmentAmountForeignCurrency": lambda n : setattr(self, 'previous_recoupment_amount_foreign_currency', n.get_float_value()),
            "previousRetained": lambda n : setattr(self, 'previous_retained', n.get_float_value()),
            "previousUnitPrice": lambda n : setattr(self, 'previous_unit_price', n.get_float_value()),
            "quantity": lambda n : setattr(self, 'quantity', n.get_float_value()),
            "realizedGainOrLoss": lambda n : setattr(self, 'realized_gain_or_loss', n.get_float_value()),
            "recoupmentAmount": lambda n : setattr(self, 'recoupment_amount', n.get_float_value()),
            "recoupmentPercentOfCompletedWork": lambda n : setattr(self, 'recoupment_percent_of_completed_work', n.get_float_value()),
            "retainage": lambda n : setattr(self, 'retainage', n.get_float_value()),
            "status": lambda n : setattr(self, 'status', n.get_str_value()),
            "totalRetained": lambda n : setattr(self, 'total_retained', n.get_float_value()),
            "totalRetainedPercent": lambda n : setattr(self, 'total_retained_percent', n.get_float_value()),
            "totalRetentionPercent": lambda n : setattr(self, 'total_retention_percent', n.get_float_value()),
            "unit": lambda n : setattr(self, 'unit', n.get_str_value()),
            "unitPrice": lambda n : setattr(self, 'unit_price', n.get_float_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_float_value("advanceAmount", self.advance_amount)
        writer.write_float_value("advancePercent", self.advance_percent)
        writer.write_str_value("aggregateBy", self.aggregate_by)
        writer.write_float_value("amount", self.amount)
        writer.write_uuid_value("associationId", self.association_id)
        writer.write_enum_value("associationType", self.association_type)
        writer.write_str_value("budgetCode", self.budget_code)
        writer.write_str_value("budgetFormattedCode", self.budget_formatted_code)
        writer.write_uuid_value("budgetId", self.budget_id)
        writer.write_float_value("budgetTransfers", self.budget_transfers)
        writer.write_float_value("budgetTransfersQuantity", self.budget_transfers_quantity)
        writer.write_float_value("budgetTransfersUnitPrice", self.budget_transfers_unit_price)
        writer.write_bool_value("canDelete", self.can_delete)
        writer.write_str_value("changedBy", self.changed_by)
        writer.write_float_value("claimedAmount", self.claimed_amount)
        writer.write_float_value("claimedQuantity", self.claimed_quantity)
        writer.write_float_value("claimedUnitPrice", self.claimed_unit_price)
        writer.write_float_value("completedWorkReleased", self.completed_work_released)
        writer.write_float_value("completedWorkRetained", self.completed_work_retained)
        writer.write_float_value("completedWorkRetainedPercent", self.completed_work_retained_percent)
        writer.write_float_value("completedWorkRetention", self.completed_work_retention)
        writer.write_float_value("completedWorkRetentionPercent", self.completed_work_retention_percent)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("creatorId", self.creator_id)
        writer.write_float_value("currentCompletedWorkRetention", self.current_completed_work_retention)
        writer.write_float_value("currentCompletedWorkRetentionForeignCurrency", self.current_completed_work_retention_foreign_currency)
        writer.write_float_value("currentCompletedWorkRetentionPercent", self.current_completed_work_retention_percent)
        writer.write_float_value("currentMaterialsRetention", self.current_materials_retention)
        writer.write_float_value("currentMaterialsRetentionForeignCurrency", self.current_materials_retention_foreign_currency)
        writer.write_float_value("currentMaterialsRetentionPercent", self.current_materials_retention_percent)
        writer.write_float_value("currentReleased", self.current_released)
        writer.write_float_value("currentRetention", self.current_retention)
        writer.write_str_value("description", self.description)
        writer.write_float_value("exchangeRate", self.exchange_rate)
        writer.write_str_value("externalId", self.external_id)
        writer.write_str_value("externalMessage", self.external_message)
        writer.write_str_value("externalSystem", self.external_system)
        writer.write_bool_value("hasComment", self.has_comment)
        writer.write_uuid_value("id", self.id)
        writer.write_str_value("integrationState", self.integration_state)
        writer.write_str_value("integrationStateChangedAt", self.integration_state_changed_at)
        writer.write_str_value("integrationStateChangedBy", self.integration_state_changed_by)
        writer.write_bool_value("isPrivate", self.is_private)
        writer.write_str_value("lastReviewedBy", self.last_reviewed_by)
        writer.write_datetime_value("lastSyncTime", self.last_sync_time)
        writer.write_float_value("materials", self.materials)
        writer.write_float_value("materialsBilled", self.materials_billed)
        writer.write_float_value("materialsBilledQuantity", self.materials_billed_quantity)
        writer.write_str_value("materialsBilledUnit", self.materials_billed_unit)
        writer.write_float_value("materialsBilledUnitPrice", self.materials_billed_unit_price)
        writer.write_float_value("materialsOnStore", self.materials_on_store)
        writer.write_float_value("materialsOnStoreQuantity", self.materials_on_store_quantity)
        writer.write_float_value("materialsOnStoreReleased", self.materials_on_store_released)
        writer.write_float_value("materialsOnStoreRetentionPercent", self.materials_on_store_retention_percent)
        writer.write_str_value("materialsOnStoreUnit", self.materials_on_store_unit)
        writer.write_float_value("materialsOnStoreUnitPrice", self.materials_on_store_unit_price)
        writer.write_float_value("materialsReleased", self.materials_released)
        writer.write_float_value("materialsRetained", self.materials_retained)
        writer.write_float_value("materialsRetainedPercent", self.materials_retained_percent)
        writer.write_float_value("materialsRetention", self.materials_retention)
        writer.write_float_value("materialsRetentionPercent", self.materials_retention_percent)
        writer.write_str_value("name", self.name)
        writer.write_float_value("netAmount", self.net_amount)
        writer.write_float_value("netAmountForeignCurrency", self.net_amount_foreign_currency)
        writer.write_float_value("originalAmount", self.original_amount)
        writer.write_float_value("originalExchangeRate", self.original_exchange_rate)
        writer.write_float_value("originalQuantity", self.original_quantity)
        writer.write_float_value("originalUnitPrice", self.original_unit_price)
        writer.write_uuid_value("parentId", self.parent_id)
        writer.write_uuid_value("paymentId", self.payment_id)
        writer.write_int_value("position", self.position)
        writer.write_float_value("previousAdvanceAmount", self.previous_advance_amount)
        writer.write_float_value("previousAdvanceAmountForeignCurrency", self.previous_advance_amount_foreign_currency)
        writer.write_float_value("previousAmount", self.previous_amount)
        writer.write_float_value("previousAmountForeignCurrency", self.previous_amount_foreign_currency)
        writer.write_float_value("previousClaimedAmount", self.previous_claimed_amount)
        writer.write_float_value("previousClaimedAmountForeignCurrency", self.previous_claimed_amount_foreign_currency)
        writer.write_float_value("previousClaimedQuantity", self.previous_claimed_quantity)
        writer.write_float_value("previousClaimedUnitPrice", self.previous_claimed_unit_price)
        writer.write_float_value("previousCompletedWorkRetained", self.previous_completed_work_retained)
        writer.write_float_value("previousCompletedWorkRetainedForeignCurrency", self.previous_completed_work_retained_foreign_currency)
        writer.write_float_value("previousExchangeRate", self.previous_exchange_rate)
        writer.write_float_value("previousMaterials", self.previous_materials)
        writer.write_float_value("previousMaterialsBilled", self.previous_materials_billed)
        writer.write_float_value("previousMaterialsBilledForeignCurrency", self.previous_materials_billed_foreign_currency)
        writer.write_float_value("previousMaterialsBilledQuantity", self.previous_materials_billed_quantity)
        writer.write_str_value("previousMaterialsBilledUnit", self.previous_materials_billed_unit)
        writer.write_float_value("previousMaterialsBilledUnitPrice", self.previous_materials_billed_unit_price)
        writer.write_float_value("previousMaterialsOnStore", self.previous_materials_on_store)
        writer.write_float_value("previousMaterialsRetained", self.previous_materials_retained)
        writer.write_float_value("previousMaterialsRetainedForeignCurrency", self.previous_materials_retained_foreign_currency)
        writer.write_float_value("previousNetAmount", self.previous_net_amount)
        writer.write_float_value("previousQuantity", self.previous_quantity)
        writer.write_float_value("previousRecoupmentAmount", self.previous_recoupment_amount)
        writer.write_float_value("previousRecoupmentAmountForeignCurrency", self.previous_recoupment_amount_foreign_currency)
        writer.write_float_value("previousRetained", self.previous_retained)
        writer.write_float_value("previousUnitPrice", self.previous_unit_price)
        writer.write_float_value("quantity", self.quantity)
        writer.write_float_value("realizedGainOrLoss", self.realized_gain_or_loss)
        writer.write_float_value("recoupmentAmount", self.recoupment_amount)
        writer.write_float_value("recoupmentPercentOfCompletedWork", self.recoupment_percent_of_completed_work)
        writer.write_float_value("retainage", self.retainage)
        writer.write_str_value("status", self.status)
        writer.write_float_value("totalRetained", self.total_retained)
        writer.write_float_value("totalRetainedPercent", self.total_retained_percent)
        writer.write_float_value("totalRetentionPercent", self.total_retention_percent)
        writer.write_str_value("unit", self.unit)
        writer.write_float_value("unitPrice", self.unit_price)
        writer.write_datetime_value("updatedAt", self.updated_at)
    




from enum import Enum

class WithIssueGetResponse_status(str, Enum):
    Draft = "draft",
    Open = "open",
    Pending = "pending",
    In_progress = "in_progress",
    Completed = "completed",
    In_review = "in_review",
    Not_approved = "not_approved",
    In_dispute = "in_dispute",
    Closed = "closed",




from enum import Enum

class WithItemIdTransitionPostRequestBody_stateId(str, Enum):
    PermittedActions = "permittedActions",
    Draft = "draft",
    Rev = "rev",
    Sbc = "sbc",
    Mgr = "mgr",
    Void = "void",

